
/**************************************
   Gestione della connessione in rete
 **************************************/

#include <FL/Fl.h>

class NetIO {
	/**-- INTERFACCIA PUBBLICA DI Users --**/
	public:
	NetIO();
	~NetIO();

	void connetti( char *server, unsigned short porta );
	int invia( char *fmt, ... );
	void set_parser( void (*)(char*) );
	/**-----------------------------------**/

	private:
#define BUFFSIZE 10240	// Dimensione del buffer di ricezione

	FILE *in, *out;
	int s;

	char *recv_buffer;		// Puntatore al buffer
	char *recv_pos;			// Puntatore alla fine del buffer
	
	void (*parser)(char *);		// Puntatore alla funzione di parsing
					// del protocollo IRC.

	static void receive_data_callback( int, void * );
	void receive( void );
	int elabora_coda( void );

};

NetIO netio;

NetIO::NetIO()
{
	/* Alloca il buffer di ricezione */
	recv_buffer = (char*)malloc(BUFFSIZE);

	/* Inizialimente il buffer e' vuoto */
	recv_pos = recv_buffer;
	recv_buffer[0] = 0;
}

NetIO::~NetIO()
{
	free( recv_buffer );
}

void NetIO::connetti( char *server, unsigned short porta )
{
    	struct sockaddr_in address;
    	struct hostent *hp;

	/* Crea una nuova socket */
	s = socket(PF_INET, SOCK_STREAM, 0);
	if (s == -1)
	{
		perror("socket()");
        	exit(1);
	}
    
	/* Tenta di risolvere il nome del server */
	if ( (hp = gethostbyname(server)) == NULL)
	{
        	perror("gethostbyname()");             
        	exit(1);
    	}

	/* Imposta la struttura sockaddr con l'indirizzo e la porta del server */
    	memset(&address,0,sizeof(address));
    	memcpy((char *)&address.sin_addr,hp->h_addr,hp->h_length);
    	address.sin_family = AF_INET;
    	address.sin_port= htons((u_short)porta);

	/* Connette la socket col server */
    	if ( (connect(s, (struct sockaddr *) &address, sizeof(address))) == -1 )
    	{
        	perror("connect()");
        	exit(1);
    	}

	/* Bind con l'handler delle FLTK */
	Fl::add_fd( s, receive_data_callback, this );
}


int NetIO::invia( char *fmt, ... )
{
	char buff[1025];
	va_list args;

	/* Formatta la stringa passata come parametri */
	va_start( args, fmt );
	vsnprintf( buff, 1024, fmt, args );
	va_end( args );
	buff[1024] = '\0';

	/* Invia la stringa al server */
	return send( s, buff, strlen(buff), 0 );
}

void NetIO::receive_data_callback( int, void *netio_callback )
{
	((NetIO*)netio_callback)->receive();
}

void NetIO::set_parser( void (*p)(char*) )
{
	parser = p;
}

void NetIO::receive( void )
{
	
	int l;
	l = recv( s, recv_pos, BUFFSIZE - (recv_pos - recv_buffer), 0 );
	recv_pos += l;
	*recv_pos = 0;

	while ( elabora_coda() );	
}

int NetIO::elabora_coda( void )
{
	char *end;

	end = strstr( recv_buffer, "\n" );
	if ( !end ) return 0;

	*end = 0;
	--end;
	if ( *end == '\r' )
		*end=0;
	++end;

	parser( recv_buffer );

	memcpy( recv_buffer, end+1, recv_pos-(end+1) );
	recv_pos -= (end+1)-recv_buffer;

	return 1;
}



