
/*******************************
   Gestione della lista utenti
 *******************************/

#include <stdio.h>
#include <FL/Fl_Hold_Browser.h> 

class Users
{
	/**-- INTERFACCIA PUBBLICA DI Users --**/
	public:
	Users( Fl_Hold_Browser *fl_widget );
	~Users();
	
	void add( const char *nick );
	void del( const char *nick );
	void rename( const char *from, const char *to );
	void clear( void );

	int count( void );
	void setop( const char *nick );
	void unsetop( const char *nick );
	int isop( const char *nick );
	void setvoice( const char *nick );
	void unsetvoice( const char *nick );
	int isvoice( const char *nick );

	void refresh( void );
	void selected( char *buff );
	char *selected( void );
	/**-----------------------------------**/
	
	private:
	struct user {
		char nick[20];
		int op;
		int voice;
	} utenti[255];
	int countutenti;

	Fl_Hold_Browser *browser_utenti;

	int find( const char *nick );
	void sort( void );
	static void browser_callback( Fl_Widget *widget, void * );
	int less( struct user a, struct user b );
};

Users::Users( Fl_Hold_Browser *fl_widget ) : countutenti(0), browser_utenti(fl_widget)
{
	browser_utenti->callback(browser_callback);
}

Users::~Users()
{ /* VUOTO */ }

void Users::browser_callback( Fl_Widget *widget, void * )
{
}

int Users::find( const char *nick )
{
	int i;
	
	for ( i=0; i<countutenti; i++ )
		if ( strcmp(nick, utenti[i].nick)==0 ) return i;

	return -1;
}

void Users::add( const char *nick )
{
	strcpy( utenti[countutenti].nick, nick );
	utenti[countutenti].op = 0;
	utenti[countutenti].voice = 0;
	countutenti++;
}

void Users::clear( void )
{
	countutenti=0;
}

void Users::del( const char *nick )
{
	int i;
	
	/* Cerca il nick nella lista */
	i = find(nick);

	/* Se non c'e' nella lista esci. */
	if ( i==-1 ) return;

	/* Elimina il nick dalla lista */
	for ( ; i<countutenti-1; i++ )
		utenti[i] = utenti[i+1];

	countutenti--;
}

void Users::rename( const char *from, const char *to )
{
	int i;
	i = find(from);
	if ( i==-1 ) return;
	strcpy( utenti[i].nick, to );
}

void Users::setop( const char *nick )
{
	int i;
	i = find(nick);
	if ( i==-1 ) return;
	utenti[i].op=1;
}
void Users::unsetop( const char *nick )
{
	int i;
	i = find(nick);
	if ( i==-1 ) return;
	utenti[i].op=0;
}
int Users::isop( const char *nick )
{
	int i;
	i = find(nick);
	if ( i==-1 ) return -1;
	return utenti[i].op;
}

void Users::setvoice( const char *nick )
{
	int i;
	i = find(nick);
	if ( i==-1 ) return;
	utenti[i].voice=1;
}
void Users::unsetvoice( const char *nick )
{
	int i;
	i = find(nick);
	if ( i==-1 ) return;
	utenti[i].voice=0;
}
int Users::isvoice( const char *nick )
{
	int i;
	i = find(nick);
	if ( i==-1 ) return -1;
	return utenti[i].voice;
}

int Users::count( void )
{
	return countutenti;
}

void Users::refresh( void )
{
	sort();
	browser_utenti->clear();

	for ( int i=0; i<countutenti; i++ )
	{
		char n[30];
		n[0] = '\0';
		if ( utenti[i].op )
			strcat( n, "@C180" );
		if ( utenti[i].voice ) 
			strcat( n, "@C60" );
		
		strcat( n, utenti[i].nick );
		browser_utenti->add( n );
	}

}

void Users::sort( void )
{
	struct user t;

	for ( int i=countutenti; i>1; i-- )
	{
		int finito=1;
		for ( int j=0; j<i-1; j++ )
			if ( less( utenti[j], utenti[j+1] ) )
			{
				t = utenti[j];
				utenti[j] = utenti[j+1];
				utenti[j+1] = t;
				finito = 0;
			}
		if ( finito ) return;
	}
}

int Users::less( struct user a, struct user b )
{
	if ( a.op < b.op ) return 1;
	if ( a.op > b.op ) return 0;
	if ( strcasecmp(a.nick,b.nick) > 0 ) return 1;
	return 0;
}

void Users::selected( char *buff )
{
	if ( !browser_utenti->value() ) {
		buff[0] = '\0';
		return;
	}

	strcpy( buff, utenti[browser_utenti->value()-1].nick );
}

char *Users::selected( void )
{
static	char buff[20];
	selected( buff );
	return buff;
}

